#!/usr/bin/env python3

import os
import sys
import time
import asyncio

class text:
    PURPLE = '\033[95m'
    CYAN = '\033[96m'
    DARKCYAN = '\033[36m'
    BLUE = '\033[94m'
    GREEN = '\033[92m'
    YELLOW = '\033[93m'
    RED = '\033[91m'
    BOLD = '\033[1m'
    UNDERLINE = '\033[4m'
    END = '\033[0m'

# User input config
os.system('clear')
filename = input('Enter path to the input file: ')
limit = input('Time limit (M:SS, blank for none): ')
end_time = 0
use_limit = False
if len(limit) > 0:
    try:
        use_limit = True
        limit = [int(x) for x in limit.split(':')]
        if limit[1] > 59:
            raise ValueError
        limit = limit[0]*60 + limit[1]
    except ValueError:
        print('Bad format, using no time limit!')
        sys.stdout.flush()
        time.sleep(2)

field_in = []

# Load initial play field
with open(filename) as f:
    for line in f:
        field_in.append([int(x) for x in line if x != '\n'])

WIDTH = 8
HEIGHT = 8
pos_x = 0
pos_y = 0
last_pos_x = 0
last_pos_y = 0
selected = False
score = 0
field = field_in[:HEIGHT]
fill_pos = [0]*8

# Draw whole field
def draw():
    os.system('clear')
    # Print score and time limit if set
    print('Score: {}'.format(score))
    if use_limit:
        remain = end_time - int(time.time())
        if remain <= 0:
            print('Time is up! Score: {}'.format(score))
            exit()
        print('Remaining time: {}:{:02d}'.format(int(remain / 60), remain % 60))

    for y in range(-1, HEIGHT):
        for x in range(WIDTH):
            color = ''
            char = 'O'
            if y == -1:
                val = field_in[x+8][fill_pos[x]]
            else:
                val = field[y][x]
       
            if val == 0:
                char = ' '
            if val == 1:
                color = text.RED
            elif val == 2:
                color = text.GREEN
            elif val == 3:
                color = text.CYAN
            elif val == 5:
                color = text.YELLOW
            elif val == 6:
                color = text.PURPLE

            if x == pos_x and y == pos_y:
                color += text.UNDERLINE
                if selected:
                    color += text.BOLD

            print(color + char + text.END, end='')
        print()
        if y == -1:
            print('--------')

# Field search for triples
def search():
    global score
    triples = [] # [x start, y start, 0=horizontal/1=vertical]

    for y in range(HEIGHT):
        for x in range(WIDTH):
            if x < WIDTH - 2 and field[y][x] == field[y][x+1] and field[y][x] == field[y][x+2]:
                triples.append([x, y, 0])
            if y < HEIGHT - 2 and field[y][x] == field[y+1][x] and field[y][x] == field[y+2][x]:
                triples.append([x, y, 1])

    # No triples were found
    if len(triples) == 0:
        return False

    # If triples were found, add score and remove gems
    score += len(triples) * 3
    for t in triples:
        field[t[1]][t[0]] = 0
        if t[2] == 0:
            field[t[1]][t[0]+1] = 0
            field[t[1]][t[0]+2] = 0
        else:
            field[t[1]+1][t[0]] = 0
            field[t[1]+2][t[0]] = 0

    # Gravity
    falling = True
    while falling:
        falling = False
        for y in range(HEIGHT - 1, -1, -1):
            for x in range(WIDTH):
                if field[y][x] == 0:
                    falling = True
                    if y > 0:
                        field[y][x] = field[y-1][x]
                        field[y-1][x] = 0
                    else:
                        field[y][x] = field_in[x+8][fill_pos[x]]
                        fill_pos[x] += 1
                        if fill_pos[x] >= len(field_in[x+8]):
                            fill_pos[x] = 0
        draw()
        sys.stdout.flush()
        time.sleep(0.3)
    return True

async def count_time(limit):
    while limit > -1:
        print('Remaining time: {}:{}'.format(int(limit / 60), limit % 60))
        limit -= 1
        await asyncio.sleep(1)

if use_limit:
    end_time = int(time.time()) + limit
    #loop = asyncio.get_event_loop()
    #loop.run_until_complete(count_time(limit))
    #loop.close()

# Main loop
while True:
    draw()

    # User commands
    inp = input('Command (w-up, s-down, a-left, d-right, space-select, q-quit): ').lower()
    last_pos_x = pos_x
    last_pos_y = pos_y
    if inp == 'q':
        print('Game over! Score: {}'.format(score))
        exit()
    elif inp == 'w' and pos_y > 0:
        pos_y -= 1
    elif inp == 's' and pos_y < HEIGHT - 1:
        pos_y += 1
    elif inp == 'a' and pos_x > 0:
        pos_x -= 1
    elif inp == 'd' and pos_x < WIDTH - 1:
        pos_x += 1
    elif inp == ' ':
        selected = not selected

    if selected and (pos_x != last_pos_x or pos_y != last_pos_y):
        # Switch gems
        cache = field[pos_y][pos_x]
        field[pos_y][pos_x] = field[last_pos_y][last_pos_x]
        field[last_pos_y][last_pos_x] = cache
        selected = False
        # First search for triples
        found = search()
        # Return gems to initial position if not found
        if not found:
            field[last_pos_y][last_pos_x] = field[pos_y][pos_x]
            field[pos_y][pos_x] = cache

        # Continue with the search
        while search():
            pass
